#include "guimodule.h"
#include "windows.h"
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

#define PN_SHAPE 0
#define PN_FROMMOD 2

#define DEFAULT_VALUE "(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)(0.0000)";
#define NUMSLIDERS 16

GuiModule::GuiModule(seGuiCallback seaudioMaster, void *p_resvd1) : SEGUI_base(seaudioMaster, p_resvd1)
{
	CallHost(seGuiHostSetWindowType, 0 );
	CallHost(seGuiHostSetWindowSize, 100, 100 );
	CallHost(seGuiHostSetWindowSizeable, 1 );
}

void GuiModule::Initialise(bool loaded_from_file)
{
	SEGUI_base::Initialise(loaded_from_file);

	which_drag = -1;
//	index_from_module = -1;

	pos_array = new float [ NUMSLIDERS * sizeof(float) ];

	OnValueChanged();
}

GuiModule::~GuiModule()
{
	delete pos_array;
}

void GuiModule::OnGuiPinValueChange(SeGuiPin *p_pin)
{
	switch( p_pin->getIndex() )
	{
		case PN_SHAPE:			
			OnValueChanged();
			break;
		case PN_FROMMOD:
			InvalidateControl();
		default:
			return;
	}
}

void GuiModule::OnValueChanged()
{
	if( GetValueS().empty() ) // set defaults
	{
		SetValueS( DefaultValue() );
		return; // rely on recursion to re-call this routine
	}

//	if( !m_inhibit_update )
//	{
		UpdatePositions( pos_array, GetValueS() );
		InvalidateControl();
//	}
}

void GuiModule::OnLButtonDown( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	SetCapture(wi);
	OnMouseMove( wi, nFlags, point );
}

void GuiModule::OnMouseMove( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	if( !GetCapture(wi) )
		return;

	int ctl_width = wi->width;

	if ((ctl_width % 2) != 0)	
	{
		ctl_width -= 1;	
	}

	int ctl_height = wi->height;
	int mid_y = ctl_height/2;
	int sl_width = ctl_width/NUMSLIDERS;
	int x_pos = point.x;
	int y_pos = point.y;
	x_pos = max(x_pos,0);
	x_pos = min(x_pos,ctl_width);
	y_pos = max(y_pos,0);
	y_pos = min(y_pos,ctl_height);
	
	for(int i = 0; i < NUMSLIDERS; i++)
	{
		if ( (x_pos > i * sl_width) && (x_pos < (i + 1) * sl_width) )
		{
			which_drag = i;
			pos_array[which_drag] =  (1.f / (float) ctl_height) * y_pos;
		}
	}

	//convert array to string for patch mem/module.cpp communication
	char v[300] = "";
	for( int j = 0 ; j < NUMSLIDERS; j++ )
	{
		float val = pos_array[j];
		char val_t[20];
		sprintf(val_t,"(%6.5f)",val);
		strcat(v,val_t);
		assert( strlen(v) < 280 );
	}

	m_inhibit_update = true; // prevent jitter due to float->text->float conversion
	SetValueS( v ); // should default to current patch
	m_inhibit_update = false;
	InvalidateControl();
//	getPin(PN_OUT)->setValueFloat( pos_array[0]);
}

void GuiModule::OnLButtonUp( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	//getPin(PN_OUT)->setValueFloat( 0.f);
	if( !GetCapture(wi) )
	return;

	ReleaseCapture(wi); // get mouse moves

	// clear on-screen drag co-ords
	which_drag = -1;
	InvalidateControl();
}

void GuiModule::paint( HDC hDC, SEWndInfo *wi )
{

	int ctl_width = wi->width;

	if ((ctl_width % 2) != 0)	
	{
		ctl_width -= 1;	
	}

	int sl_width = ctl_width/NUMSLIDERS;
	int ctl_height = wi->height;
//	int mid_y = ctl_height/2;

	// Fill in solid background black
	HBRUSH background_brush = CreateSolidBrush( RGB(0,255,0) );
	HBRUSH highlight_brush = CreateSolidBrush( RGB(255,0,0) );

	for(int i = 0; i < NUMSLIDERS; i++)
	{
		int position = (float) ctl_height * pos_array[i];
		position = max(position,1);
		position = min(position,ctl_height - 2);

		RECT r;
		r.top = position - 1;
		r.left = i * sl_width;
		r.right = (i + 1) * sl_width;
		//r.bottom = position + 2;
		r.bottom = ctl_height;
		if(i == getPin(PN_FROMMOD)->getValueInt()/*index_from_module*/ )
		{
			FillRect( hDC, &r, highlight_brush );
		}
		else FillRect( hDC, &r, background_brush );

	}

	// cleanup objects
    DeleteObject(background_brush);
	DeleteObject(highlight_brush);

  }

int GuiModule::InvalidateControl(void)
{
	return CallHost(seGuiHostRequestRepaint);
}

SeSdkString GuiModule::GetValueS()
{
	return getPin(PN_SHAPE)->getValueText();
}

void GuiModule::SetValueS(SeSdkString p_string)
{
	getPin(PN_SHAPE)->setValueText(p_string);
}

SeSdkString GuiModule::DefaultValue()
{
	return DEFAULT_VALUE;
}

// convert string value into segment list
void GuiModule::UpdatePositions( float *pos_array, const SeSdkString &p_values)
{
	SeSdkString s(p_values);

	// convert CString of numbers to array of screen co-ords
	int i = 0;
	while( s.length() > 0 && i < NUMSLIDERS )
	{
		int p1 = s.find("(");
		int p2 = s.find(")");
		if( p2 < 1 )
			return;

		float slider_val = StringToFloat( s.substr(p1+1,p2-1) );// Check this!
		pos_array[i] = slider_val;

		i++;

		p2++;
		s = s.substr(p2, s.length() - p2);
	}

	if( i == 0 ) // if string empty, set defaults
	{
		SetValueS( DefaultValue() );		
	}

}

float GuiModule::StringToFloat(const SeSdkString &p_string)
{
	char * temp;	// convert string to float
	return (float) strtod( p_string.data(), &temp );
}
/*
void GuiModule::OnModuleMsg(int p_user_msg_id,int p_length, void *p_data)
{
    if (p_user_msg_id == id_to_int('o','m','o','d'))
	{
        index_from_module = atoi ( (const char *)p_data );
		InvalidateControl();
	}
}
*/