#include "guimodule.h"
#include "windows.h"
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <math.h>

#define PN_SEED 0
#define PN_FILL 1
#define PN_RANDOMISE 2
#define PN_ARRAY 3
#define PN_STEPS 4
#define PN_GAP 5
#define PN_DEBUG 6
#define NUMSLIDERS 16
#define DEFAULT_VALUE 0

GuiModule::GuiModule(seGuiCallback seaudioMaster, void *p_resvd1) : SEGUI_base(seaudioMaster, p_resvd1)
,m_inhibit_update(false)
{
	CallHost(seGuiHostSetWindowType, 0 );
	CallHost(seGuiHostSetWindowSize, 100, 100 );
	CallHost(seGuiHostSetWindowSizeable, 1 );
}

void GuiModule::Initialise(bool loaded_from_file)
{
	SEGUI_base::Initialise(loaded_from_file);

	pos_array = new int [ NUMSLIDERS * sizeof(int) ];

	if(!loaded_from_file)
	{
		getPin(PN_SEED)->setValueInt(5);
		getPin(PN_STEPS)->setValueInt(11);
		getPin(PN_GAP)->setValueInt(0);
	}

	OnValueChanged();
}

GuiModule::~GuiModule()
{
	delete pos_array;
}

void GuiModule::OnGuiPinValueChange(SeGuiPin *p_pin)
{
	switch( p_pin->getIndex() )
	{
		case PN_SEED:
		{
			srand ( getPin(PN_SEED)->getValueInt() );
			break;
		}
		case PN_FILL:
		{
			InvalidateControl();
			break;
		}
		case PN_GAP:
		{
			InvalidateControl();
			break;
		}
		case PN_STEPS:
		{
			InvalidateControl();
			break;
		}
		case PN_RANDOMISE:
		{
			if (getPin(PN_RANDOMISE)->getValueInt() == 1)
			{
				char v[300] = "";
				for( int i = 0 ; i < NUMSLIDERS; i++ )
				{
					char cell[20] = "";
					float r = (float) rand()/RAND_MAX;

					int o;
					r = r * getPin(PN_STEPS)->getValueInt();
					o = (int) r;

					pos_array[i] = o;
					sprintf(cell,"%i,", o);
					strcat(v,cell);
					assert( strlen(v) < 280 );
				}
				
				getPin(PN_ARRAY)->setValueText( v );

				InvalidateControl();
			}
			break;		
		}
		case PN_ARRAY:
		{
			OnValueChanged();
			break;	
		}
	}
	
}

void GuiModule::parseArrayString( const SeSdkString &p_values )	
{
	SeSdkString s(p_values);

	int i = 0;
	while( s.length() > 0 && i < NUMSLIDERS )
	{
		int comma = s.find(",");

		int val = (int) StringToFloat( s.substr(comma-1,1) );

		pos_array[i] = val;

		i++;
		comma++;
		s = s.substr(comma, s.length() - comma);
	}
}

float GuiModule::StringToFloat(const SeSdkString &p_string)
{
	char * temp;	// convert string to float
	return (float) strtod( p_string.data(), &temp );
}

//Mouse
void GuiModule::OnLButtonDown( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	SetCapture(wi);
	OnMouseMove( wi, nFlags, point );
}

void GuiModule::OnMouseMove( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	if( !GetCapture(wi) )
		return;

	int ctl_width = wi->width;

	if ((ctl_width % 2) != 0)	
	{
		ctl_width -= 1;	
	}

	int ctl_height = wi->height;
	int mid_y = ctl_height/2;
	int sl_width = ctl_width/NUMSLIDERS;
	int x_pos = point.x;
	int y_pos = point.y;
	x_pos = max(x_pos,0);
	x_pos = min(x_pos,ctl_width);
	y_pos = max(y_pos,0);
	y_pos = min(y_pos,ctl_height);
	int steps = getPin(PN_STEPS)->getValueInt();
	
	for(int i = 0; i < NUMSLIDERS; i++)
	{
		if ( (x_pos > i * sl_width) && (x_pos < (i + 1) * sl_width) )
		{
			which_drag = i;
			float temp = ((float) y_pos / (float) ctl_height) * (steps-1);
			//debug
			char test[20];
			sprintf(test,"%i",ftoi(temp));
			getPin(PN_DEBUG)->setValueText(test);
			////////////////////////////////////////

			pos_array[which_drag] = ftoi(temp);
		}
	}

	char v[300] = "";
	for( int j = 0 ; j < NUMSLIDERS; j++ )
	{
		int val = pos_array[j];
		char cell[20] = "";
		sprintf(cell,"%i,",val);
		strcat(v,cell);
		assert( strlen(v) < 280 );
	}

	m_inhibit_update = true; 
	SetValueS( v ); 
	m_inhibit_update = false;
	InvalidateControl();
}

void GuiModule::OnLButtonUp( SEWndInfo *wi, UINT nFlags, sepoint point )
{
	if( !GetCapture(wi) )
	return;

	ReleaseCapture(wi); // get mouse moves

	// clear on-screen drag co-ords
	which_drag = -1;
	InvalidateControl();
}

//Paint
void GuiModule::paint( HDC hDC, SEWndInfo *wi )
{
	int ctl_width = wi->width;

	if ((ctl_width % 2) != 0)	
	{
		ctl_width -= 1;	
	}

	int sl_width = ctl_width/NUMSLIDERS;
	int ctl_height = wi->height;
	int steps = getPin(PN_STEPS)->getValueInt(); 
	int gap = getPin(PN_GAP)->getValueInt();
	int blockheight = ctl_height / steps;

	

	HBRUSH background_brush = CreateSolidBrush( getPin(PN_FILL)->getValueInt() ); 
	//HBRUSH highlight_brush = CreateSolidBrush( RGB(255,0,0) );

	for(int i = 0; i < NUMSLIDERS; i++)
	{
		int offset = pos_array[i] * blockheight;
		RECT r;
		r.top = offset;
		r.left = i * sl_width;
		r.right = ((i + 1) * sl_width) - gap;
		r.bottom = offset + blockheight;
		FillRect( hDC, &r, background_brush );
	}
	// cleanup objects
    DeleteObject(background_brush);
	//DeleteObject(highlight_brush);
}

int GuiModule::InvalidateControl(void)
{
	return CallHost(seGuiHostRequestRepaint);
}

SeSdkString GuiModule::DefaultValue()
{
	char v[300] = "";

	for( int i = 0 ; i < NUMSLIDERS; i++ )
	{
		char cell[20] = "0,";
		pos_array[i] = 0;
		strcat(v,cell);
		assert( strlen(v) < 280 );
	}
	
	return v;
}

SeSdkString GuiModule::GetValueS()
{
	return getPin(PN_ARRAY)->getValueText();
}

void GuiModule::SetValueS(SeSdkString p_string)
{
	getPin(PN_ARRAY)->setValueText(p_string);
}

void GuiModule::OnValueChanged()
{
	if( GetValueS().empty() ) // set a default straight line
	{
		SetValueS( DefaultValue() );
		return; // rely on recursion to re-call this routine
	}

	if( !m_inhibit_update )
	{
		parseArrayString( GetValueS() );
		InvalidateControl();
	}
}

int GuiModule::ftoi(float flt)
{
    int i;
    _asm
    {
        mov  eax,flt; //loaded mem to acc
        rcl  eax,1;   //left shift acc to remove the sign
        mov  ebx,eax; //save the acc
        mov  edx,4278190080; //clear reg edx;
        and  eax,edx; //and acc to retrieve the exponent
        shr  eax,24;
        sub  eax,7fh; //subtract 7fh(127) to get the actual power 
        mov  edx,eax; //save acc val power
        mov  eax,ebx; //retrieve from ebx
        rcl  eax,8;     //trim the left 8 bits that contain the power
        mov  ebx,eax; //store
        mov  ecx, 1fh; //subtract 17 h
        sub  ecx,edx; 
        mov  edx,00000000h;
        cmp  ecx,0;
        je   loop2;
        shr  eax,1;
        or   eax,80000000h;        
loop1:    
        shr  eax,1; //shift (total bits - power bits);
        sub  ecx,1;
        add  edx,1;
        cmp  ecx,0;
        ja   loop1;
loop2:  
        mov  i, eax;        
    
//check sign +/-        
sign:
        mov  eax,flt;
        and  eax,80000000h;
        cmp  eax,80000000h;
        je     putsign;
    }

    return i;

putsign:
    return -i;
}